#!/bin/bash
# This script for install/update edge-db PostgreSQL cluster

pushd `dirname $0` > /dev/null

# Available params, ./install.sh -?
_usage() {
    echo "Usage: $0 [-V edge-version] [-v pg-version] [-d dbname] [-u username] [-p password] [-P port] [-D datadir] [-l logdir] [-b binfsdir]"
    echo " NB! For default values see config/*.config"
    echo "   -C, --config         Configuration name"
    echo "   -V, --edge-version   EdgeDB version"
    echo "   -v, --pg-version     PostgreSQL version"
    echo "   -d, --database       PostgreSQL database name"
    echo "   -u, --user           PostgreSQL user name"
    echo "   -p, --password       PostgreSQL user password"
    echo "   -P, --port           PostgreSQL port"
    echo "   -h, --host           PostgreSQL host"
    echo "   -D, --data-dir       PostgreSQL data directory"
    echo "   -l, --logs-dir       PosthreSQL logs directory"
    echo "   -b, --binfs-dir      BinFS data directory"
    echo "   -?, --help           This help text"
    exit 0
}

# Current dir
_ME=$(readlink -f "$0")
PWD=$(dirname "${_ME}")
EDBVERSION=

# Config name
[ -f ${PWD}/.configure ] && CONFIG=$(cat ${PWD}/.configure | tr -d "\n\r")
CONFIG=${CONFIG:-default}

# CLI params
while test $# -gt 0; do
    case $1 in
        --config|-C)
            CONFIG="${2}"
            shift
        ;;
        --pg-version|-v)
            PGVERSION="${2}"
            shift
        ;;
        --edge-version|-V)
            EDBVERSION="${2}"
            shift
        ;;
        --database|-d)
            PGDBNAME="${2}"
            shift
        ;;
        --user|-u)
            PGUSER="${2}"
            shift
        ;;
        --password|-p)
            PGUSERPASS="${2}"
            shift
        ;;
        --port|-P)
            PGCLUSTERPORT="${2}"
            shift
        ;;
        --host|-h)
            PGCLUSTERHOST="${2}"
            shift
        ;;
        --data-dir|-D)
            PGDATADIR="${2}"
            shift
        ;;
        --logs-dir|-l)
            PGLOGDIR="${2}"
            shift
        ;;
        --binfs-dir|-b)
            BINFSDIR="${2}"
            shift
        ;;
        --help|-?)
            _usage
            break
        ;;
        --)
            break
        ;;
        --*|-?)
            echo "Unknown option ${1}"
            exit 11
        ;;
        -*)
            split=$1
            shift
            set -- $(echo "$split" | cut -c 2- | sed 's/./-& /g') "$@"
            continue
        ;;
        *)
            echo "Unknown argument ${1}"
            exit 11
        ;;
    esac

    shift
done

# Configs
source "${PWD}/config/${CONFIG}.config"


# Helpers
_error(){
    echo "ERROR: $*" >&2
    kill $$
    exit -1
}

# Main body
if test "$EUID" -ne 0 ; then
    _error "Please run as root"
fi

# Check for contrib extensions
CT=$(dpkg -l | grep 'postgresql-[0-9]' | grep ${PGVERSION} | awk '{print "dpkg -L "$2}' | sh | egrep 'postgresql.*/extension$')
[ -d "${CT}" ] || _error "Unable to find postgresql extension dir: ${CT}"

# Check for contrib libs
CT_LIB=$(dpkg -l | grep 'postgresql-[0-9]' | grep ${PGVERSION} | awk '{print "dpkg -L "$2}' | sh | egrep 'postgresql.*/lib$')
[ -d "${CT_LIB}" ] || _error "Unable to find postgresql lib dir: ${CT_LIB}"

# Prepare temporary directory for installation process
TMP_DIR=$(mktemp -d -t __vitml.db.2.XXXXXX)
chown postgres:postgres ${TMP_DIR} && chmod 777 ${TMP_DIR}

# Override EdgeDB version if needed
if [ ! -z "${EDBVERSION}" ]; then
#    ./dumpenv.sh ${EDBVERSION}  ## todo - organize version override!
echo "## todo - organize version override!"
fi

# Set env variables
_mymajor=$(cat ${PWD}/.version.major)
_myminor=$(cat ${PWD}/.version.minor)
_myrelease=$(cat ${PWD}/.version.release)
myrevision=$(cat ${PWD}/.revision)
mybuild=$(cat ${PWD}/.build)
myrc=$(cat ${PWD}/.rc)
mypassport=$(cat ${PWD}/.passport)
myos=linux
mytz=UTC
uname -a | grep -q _64 && mymachine=x64 || mymachine=x86

# On unattended install timedatectl is not installed already, so we try to get TZ from other sources
if [ -f /usr/bin/timedatectl ]; then
    mytz=$(timedatectl status | grep "zone" | sed -e 's/^[ ]*Time zone: \(.*\) (.*)$/\1/g')
elif [ -f /etc/timezone ]; then
    mytz=$(cat /etc/timezone)
fi
# Double-check value
if [ -z "${mytz}" ]; then
    mytz=UTC
fi

export PGPORT=$PGCLUSTERPORT
export PGPASSWORD=postgres

PGDATADIR=$(psql -h $PGCLUSTERHOST -p $PGCLUSTERPORT -q -t -c "select current_setting('data_directory')" postgres postgres | awk '{print $1}')
contrib=$(cat ${PWD}/.contrib | tr "\n\r" "  ")

# Set variables for PostgreSQL
cat /dev/null > ${TMP_DIR}/vars.psql

sed 's#%%VERSION_MAJOR%%#'${_mymajor}'#g;s#%%VERSION_MINOR%%#'${_myminor}'#g;s#%%VERSION_RELEASE%%#'${_myrelease}'#g;s#%%BUILD%%#'${mybuild}'#g;s#%%REVISION%%#'${myrevision}'#g;s#%%RC%%#'${myrc}'#g;s#%%PASSPORT%%#'${mypassport}'#g;s#%%DATADIR%%#'${datadir}'#g;s#%%DBNAME%%#'${PGDBNAME}'#g;s#%%DBUSER%%#'${PGUSER}'#g;s#%%DBPASS%%#'${PGUSERPASS}'#g;s#%%CONTRIBDIR%%#'${CT}'#g;s#%%CONTRIBLIBDIR%%#'${CT_LIB}'#g;s#%%OS%%#'${myos}'#g;s#%%MACHINE%%#'${mymachine}'#g;s#%%CONTRIBLIST%%#'"${contrib}"'#g;s#%%TIMEZONE%%#'${mytz}'#g;' ${PWD}/vars.psql_ > ${TMP_DIR}/vars.psql

cat /dev/null > ${TMP_DIR}/_res

# Main install procedure
psql -h $PGCLUSTERHOST -p $PGCLUSTERPORT -q -t -v _DIR="${PWD}" -v _TMP="${TMP_DIR}" postgres postgres < ${PWD}/install.sql 2>&1 | grep ERROR\\\|FATAL > ${TMP_DIR}/_res
res=$(cat ${TMP_DIR}/_res)
if [ "X${res}" != "X" ]; then
    cat ${TMP_DIR}/_res
    _error "[FATAL] Error setting DB up "
fi

# BinFS configuration
echo "BinFS configuration..."

mkdir -p "${BINFSDIR}"
chown postgres:postgres "${BINFSDIR}"
chmod 755 "${BINFSDIR}"

export PGPASSWORD=$PGUSERPASS
cat <<EOF | psql -h $PGCLUSTERHOST -p $PGCLUSTERPORT -w $PGDBNAME $PGUSER | grep -q ERROR && _error "failed to init autocode database!"
begin;
select core.db_init();
commit;
EOF


# Cleanup
rm -rf ${TMP_DIR}

echo "ALL DONE!"

exit 0

